#ifdef GL_ES
precision highp float;
#endif

uniform sampler2D textureRgba;
uniform vec4 color;
uniform float brightness;
uniform float contrast;

varying vec2 texCoord;

vec3 rgb2hsv(vec3 c)
{
    vec4 p = mix(vec4(c.bg, vec2(-1.0, 0.66666666)), vec4(c.gb, vec2(0.0, -0.33333333)), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d)), d / (q.x + 1.0e-10), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 0.66666666, 0.33333333, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main()
{
    vec4 sample = texture2D(textureRgba, texCoord) * color;

    vec3 hsv = rgb2hsv(sample.rgb);
    hsv.z *= brightness;

    gl_FragColor = vec4(hsv2rgb(hsv), sample.a);
}
